import 'dart:async';
import 'dart:io';

import 'package:file/local.dart';
import 'package:flutter/services.dart';
import 'package:path/path.dart' as p;

enum InterruptType { BEGIN, END }

typedef OnInterrupt(InterruptType type);

class YkdAudioRecorder {
  static const MethodChannel _channel =
      const MethodChannel('ykd_audio_recorder');

  static LocalFileSystem fs = LocalFileSystem();

  bool isRecording = false;

  final OnInterrupt onInterrupt;

  YkdAudioRecorder({this.onInterrupt}) {
    _channel.setMethodCallHandler((MethodCall call) {
      if (call.method == 'interrupt' && this.onInterrupt != null) {
        String typeStr = call.arguments;
        InterruptType type =
            typeStr == 'BEGIN' ? InterruptType.BEGIN : InterruptType.END;
        this.onInterrupt(type);
      }
    });
  }

  Future<RecordStatus> start({String path}) async {
    if (path != null) {
      File file = fs.file(path);
      if (await file.exists()) {
        throw new Exception("A file already exists at the path :" + path);
      } else if (!await file.parent.exists()) {
        throw new Exception("The specified parent directory does not exist");
      }
    }
    Map<String, Object> map = Map<String, Object>.from(
        await _channel.invokeMethod('start', {'path': path}));
    isRecording = map['success'] ?? true;
    return new RecordStatus(success: map['success'], error: map['error']);
  }

  Future<RecordInfo> stop() async {
    if (isRecording) {
      Map<String, Object> map =
          Map<String, Object>.from(await _channel.invokeMethod('stop'));
      isRecording = false;
      return new RecordInfo(
          duration: new Duration(seconds: map['duration']), path: map['path']);
    }
    return null;
  }

  Future pause() async {
    return await _channel.invokeMethod('pause');
  }

  Future<double> getVolumn() async {
    return await _channel.invokeMethod('getVolumn');
  }

  Future resume() async {
    return await _channel.invokeMethod('resume');
  }
}

class RecordStatus {
  bool success;
  String error;
  RecordStatus({this.success, this.error});
}

class RecordInfo {
  String path;
  Duration duration;
  RecordInfo({this.duration, this.path});
}
